package com.xjrsoft.common.utils;

import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.xjrsoft.common.exception.MyException;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import javax.imageio.stream.FileImageOutputStream;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

import static org.apache.poi.util.LittleEndianConsts.BYTE_SIZE;

public class ImageUtil {
    public static byte[] getByteByPic(String imageUrl) throws IOException {
        File imageFile = new File(imageUrl);
        InputStream inStream = new FileInputStream(imageFile);
        BufferedInputStream bis = new BufferedInputStream(inStream);
        BufferedImage bm = ImageIO.read(bis);
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        String type = imageUrl.substring(imageUrl.length() - 3);
        ImageIO.write(bm, type, bos);
        bos.flush();
        byte[] data = bos.toByteArray();
        return data;
    }
    /**
     * 将图片压缩到指定大小以内
     *
     * @param srcImgData 源图片数据
     * @param maxSize 目的图片大小
     * @return 压缩后的图片数据
     */
    public static byte[] compressUnderSize(byte[] srcImgData, long maxSize) {
        double scale = 0.9;
        byte[] imgData = Arrays.copyOf(srcImgData, srcImgData.length);

        if (imgData.length > maxSize) {
            do {
                try {
                    imgData = compress(imgData, scale);

                } catch (IOException e) {
                    throw new IllegalStateException("压缩图片过程中出错，请及时联系管理员！", e);
                }

            } while (imgData.length > maxSize);
        }

        return imgData;
    }

    /**
     * 按照 宽高 比例压缩
     *
     * @param imgIs 待压缩图片输入流
     * @param scale 压缩刻度
     * @param out 输出
     * @return 压缩后图片数据
     * @throws IOException 压缩图片过程中出错
     */
    public static byte[] compress(byte[] srcImgData, double scale) throws IOException {
        BufferedImage bi = ImageIO.read(new ByteArrayInputStream(srcImgData));
        int width = (int) (bi.getWidth() * scale); // 源图宽度
        int height = (int) (bi.getHeight() * scale); // 源图高度

        Image image = bi.getScaledInstance(width, height, Image.SCALE_SMOOTH);
        BufferedImage tag = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        Graphics g = tag.getGraphics();
        g.setColor(Color.RED);
        g.drawImage(image, 0, 0, null); // 绘制处理后的图
        g.dispose();

        ByteArrayOutputStream bOut = new ByteArrayOutputStream();
        ImageIO.write(tag, "JPEG", bOut);

        return bOut.toByteArray();
    }

    //byte数组到图片
    public static void byte2image(byte[] data,String path){
        if(data.length<3||path.equals("")) return;
        try{
            FileImageOutputStream imageOutput = new FileImageOutputStream(new File(path));
            imageOutput.write(data, 0, data.length);
            imageOutput.close();
            System.out.println("Make Picture success,Please find image in " + path);
        } catch(Exception ex) {
            System.out.println("Exception: " + ex);
            ex.printStackTrace();
        }
    }

    public static String bytesEncode2Base64(byte[] bytes) {
        return new String(Base64.getEncoder().encode(bytes), StandardCharsets.UTF_8);
    }

    // base64图片去掉前缀
    public static String base64RemovePrefix(String baseStr) {
        return baseStr.replace("data:image/jpg;base64,", "")
                .replace("data:image/jpeg;base64,", "")
                .replace("data:image/png;base64,", "");
    }

    // 图片处理
    public static String ImageHandler(@NotNull MultipartFile file) throws IOException {
        String filename = file.getOriginalFilename();
        String suffix = StringUtils.substringAfterLast(filename, StringPool.DOT);
        String[] imgSuffix = new String[]{"png", "jpg", "jpeg"};

        if (!Arrays.asList(imgSuffix).contains(suffix)) {
            throw new MyException("图片格式不正确！");
        }
        Map<String, String> map = new HashMap<String, String>() {
            {
                put("jpg", "data:image/jpg;base64,");
                put("jpeg", "data:image/jpeg;base64,");
                put("png", "data:image/png;base64,");
            }
        };

        Long maxSize = 1048576L; // 最大文件1M
        byte[] resultImg = ImageUtil.compressUnderSize(file.getBytes(), maxSize);

        return map.get(suffix) + ImageUtil.bytesEncode2Base64(resultImg);
    }

    public static byte[] dealPicture(MultipartFile file, Integer anInt) {
        byte[] fileByte;
        try {
            byte[] imageBytes = file.getBytes();
            fileByte = ImageUtil.compressUnderSize(imageBytes, BYTE_SIZE);
            InputStream input = new ByteArrayInputStream(fileByte);
            BufferedImage image = ImageIO.read(input);
            BufferedImage rotatedImage = PictureUtil.rotatePhonePhoto(image, anInt);

            fileByte = PictureUtil.convertBufferedImageToByteArray(rotatedImage);
        } catch (Exception e) {
            throw new MyException("图片格式不符");
        }

        if (ObjectUtils.isEmpty(fileByte) || fileByte.length == 0) {
            throw new MyException("图片格式不符");
        }

        return fileByte;
    }

}
