-- 教研培训
DROP TABLE IF EXISTS `wf_teacher_training`;
CREATE TABLE `wf_teacher_training`
(
    id               bigint       not null comment '主键编号'
        primary key,
    create_user_id   bigint       null comment '创建人',
    create_date      datetime     null comment '创建时间',
    modify_user_id   bigint       null comment '修改人',
    modify_date      datetime     null comment '修改时间',
    delete_mark      int          not null comment '删除标记',
    enabled_mark     int          not null comment '有效标志',
    sort_code        int          null comment '序号',
    training_name    varchar(256) null comment '培训活动名称',
    training_topic   text         null comment '培训活动主题',
    person_in_charge varchar(256) null comment '培训负责人',
    start_time       datetime     null comment '培训开始时间',
    end_time         datetime     null comment '培训结束时间',
    training_address varchar(512) null comment '培训地点',
    `remark`         VARCHAR(1000)         DEFAULT NULL COMMENT '备注',
    `status`         INT          NOT NULL DEFAULT '0' COMMENT '状态（1:结束 0:未结束）'
)
    comment 'wf_teacher_training';

-- 教研培训参与培训人员
DROP TABLE IF EXISTS `wf_teacher_training_participant`;
CREATE TABLE `wf_teacher_training_participant`
(
    id                     bigint       not null comment '主键编号'
        primary key,
    create_user_id         bigint       null comment '创建人',
    create_date            datetime     null comment '创建时间',
    modify_user_id         bigint       null comment '修改人',
    modify_date            datetime     null comment '修改时间',
    delete_mark            int          not null comment '删除标记',
    enabled_mark           int          not null comment '有效标志',
    sort_code              int          null comment '序号',
    wf_teacher_training_id bigint       null comment '教研培训主键id（wf_teacher_training）',
    participant_id         bigint       null comment '参与培训人id',
    participant_username   varchar(256) null comment '参与培训人oa',
    participant_name       varchar(256) null comment '参与培训人姓名'
)
    comment '教研培训参与培训人员';

-- 校内实训基地数据项
DROP TABLE IF EXISTS `school_training_base`;
CREATE TABLE school_training_base
(
    id                     bigint        not null comment '主键编号'
        primary key,
    create_user_id         bigint        null comment '创建人',
    create_date            datetime      null comment '创建时间',
    modify_user_id         bigint        null comment '修改人',
    modify_date            datetime      null comment '修改时间',
    delete_mark            int           not null comment '删除标记',
    enabled_mark           int           not null comment '有效标志',
    sort_code              int           null comment '序号',
    training_base_name     VARCHAR(255)  NOT NULL COMMENT '实训基地名称',
    training_base_number   VARCHAR(255)  null DEFAULT '0' COMMENT '实训基地编号',
    training_rooms_number  INT           NOT NULL COMMENT '实训室数，数字标识，例：22',
    training_project_total INT           NOT NULL COMMENT '实训项目总数，数字标识，例：33',
    base_category          VARCHAR(255)  NOT NULL COMMENT '基地类别代码（SXJDLBDM）',
    approve_date           date          NOT NULL COMMENT '批准日期，格式：YYYYMMDD，如：2006-04-12',
    setup_year             VARCHAR(10)   NOT NULL COMMENT '成立年度，格式：YYYY，如：2006',
    base_major_id          bigint        null DEFAULT null COMMENT '面向专业',
    base_major_name        VARCHAR(256)  null DEFAULT '' COMMENT '专业名字',
    support_department     VARCHAR(255)  null DEFAULT '' COMMENT '被列为实训基地项目支持部门',
    build_area             DOUBLE(10, 2) null DEFAULT 0 COMMENT '建筑面积',
    instrument_total       INT           null DEFAULT 0 COMMENT '仪器设备总数，数字标识，例：33',
    station_number         INT           null DEFAULT 0 COMMENT '实践教学工位数，数字标识，例：33',
    manager_number_allied  INT           null DEFAULT 0 COMMENT '管理人员数（专职）',
    manager_number_part    INT           null DEFAULT 0 COMMENT '管理人员数（兼职）'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='校内实训基地数据表';

-- 学生证书表
DROP TABLE IF EXISTS `student_cert_table`;
CREATE TABLE student_cert_table
(
    id                 bigint       not null comment '主键编号'
        primary key,
    create_user_id     bigint       null comment '创建人',
    create_date        datetime     null comment '创建时间',
    modify_user_id     bigint       null comment '修改人',
    modify_date        datetime     null comment '修改时间',
    delete_mark        int          not null comment '删除标记',
    enabled_mark       int          not null comment '有效标志',
    sort_code          int          null comment '序号',
    student_id         varchar(64)  NOT NULL COMMENT '学号',
    student_name       varchar(64)  not null comment '学生姓名',
    student_gender     varchar(32)  NOT NULL COMMENT '性别',
    class_id           bigint       not null comment '学生班级id',
    class_name         varchar(64)  not null comment '学生班级',
    teacher_id         bigint       NOT NULL COMMENT '教师id',
    teacher_name       varchar(64)  NOT NULL COMMENT '教师姓名',
    cert_name          varchar(512) NOT NULL COMMENT '证书名称',
    cert_id            varchar(256) NOT NULL COMMENT '证书编号',
    cert_grade         varchar(64)  NULL COMMENT '证书等级',
    cert_organizations varchar(255) NOT NULL COMMENT '证书发放机构',
    cert_date          date         NULL COMMENT '证书发放日期',
    cert_electronic    varchar(64)  NOT NULL COMMENT '是否支持电子证书(0：否，1：是)',
    remark             varchar(1024) DEFAULT NULL COMMENT '备注',
    file_id            longtext     NOT NULL COMMENT '附件',
    status             int          NOT NULL COMMENT '状态（1:结束 0:未结束）'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='学生证书表';

-- 合作企业表
DROP TABLE IF EXISTS `company_coop`;
CREATE TABLE `company_coop`
(
    id                 bigint       not null comment '主键编号'
        primary key,
    create_user_id     bigint       null comment '创建人',
    create_date        datetime     null comment '创建时间',
    modify_user_id     bigint       null comment '修改人',
    modify_date        datetime     null comment '修改时间',
    delete_mark        int          not null comment '删除标记',
    enabled_mark       int          not null comment '有效标志',
    sort_code          int          null comment '序号',
    `company_name`     varchar(255) NOT NULL COMMENT '企业名称',
    `company_type`     varchar(64)  NOT NULL COMMENT '企业性质(企业性质字典编号)',
    `company_size`     varchar(64)  NOT NULL COMMENT '企业规模（企业规模字典标号）',
    `company_person`   varchar(255) NOT NULL COMMENT '法人代表',
    `company_position` varchar(255) NOT NULL COMMENT '企业地址',
    `contact`          varchar(255) NOT NULL COMMENT '联系人',
    `phone`            varchar(32)  NOT NULL COMMENT '联系电话',
    `partner_type`     varchar(64)  NOT NULL COMMENT '合作状态（状态类型字典编号）',
    `sector`           varchar(64)  NOT NULL COMMENT '所属行业(行业类型字典编号)',
    `industry`         varchar(64)  NOT NULL COMMENT '所属产业（产业类型字典编号）',
    `company_email`    varchar(255) NULL DEFAULT NULL COMMENT '企业邮箱',
    `contact_email`    varchar(255) NULL DEFAULT NULL COMMENT '联系人邮箱',
    `company_fax`      varchar(255) NULL DEFAULT NULL COMMENT '企业传真',
    `company_register` varchar(255) NULL DEFAULT NULL COMMENT '企业注册号',
    `company_website`  varchar(512) NULL DEFAULT NULL COMMENT '企业网址'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='合作企业表';

-- 奖助学金申请表
DROP TABLE IF EXISTS `scholarship_bursary_applications`;
CREATE TABLE `scholarship_bursary_applications`
(
    `id`                                   bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`                       bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`                          datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`                       bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`                          datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`                          int          NOT NULL COMMENT '删除标记',
    `enabled_mark`                         int          NOT NULL COMMENT '有效标志',
    `sort_code`                            int          NULL DEFAULT NULL COMMENT '序号',
    `class_name`                           varchar(256) NOT NULL COMMENT '班级',
    `teacher_id`                           bigint       NOT NULL COMMENT '班主任id',
    `teacher_name`                         varchar(256) NOT NULL COMMENT '班主任姓名',
    `student_name`                         varchar(64)  not null comment '学生姓名',
    `student_gender`                       varchar(32)  NOT NULL COMMENT '学生性别',
    `identification_number`                varchar(64)  NOT NULL COMMENT '身份证号',
    `base_semester`                        varchar(256) NOT NULL COMMENT '学期',
    `base_student_scholarship_category_id` varchar(256) NOT NULL COMMENT '申请奖助学金名称',
    `base_student_scholarship_category`    varchar(256) NOT NULL COMMENT '申请奖助学金名称',
    `personal_circumstances`               longtext     NOT NULL COMMENT '描述个人情况',
    `application_reason`                   longtext     NOT NULL COMMENT '阐述申请理由',
    `bank_name`                            varchar(256) NOT NULL COMMENT '收款银行名称',
    `bank_card_number`                     varchar(256) NOT NULL COMMENT '银行卡号',
    `bank_account_name`                    varchar(256) NOT NULL COMMENT '账户名称',
    `file_id`                              longtext     NULL COMMENT '佐证材料',
    `status`                               int          NULL DEFAULT 0 COMMENT '状态（1:结束 0:未结束）',
    PRIMARY KEY (`id`) USING BTREE
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='奖助学金申请表';












