-- 校外实训基地数据项
DROP TABLE IF EXISTS `outside_training_base`;
CREATE TABLE outside_training_base
(
    id                   bigint       not null comment '主键编号' primary key,
    create_user_id       bigint       null comment '创建人',
    create_date          datetime     null comment '创建时间',
    modify_user_id       bigint       null comment '修改人',
    modify_date          datetime     null comment '修改时间',
    delete_mark          int          not null comment '删除标记',
    enabled_mark         int          not null comment '有效标志',
    sort_code            int          null comment '序号',
    training_base_name   VARCHAR(255) NOT NULL comment '实训基地名称',
    training_base_number VARCHAR(255) null comment '实训基地编号',
    company_coop_id      bigint       NOT NULL comment '所属合作企业(company_coop)',
    company_coop_name    varchar(300) NOT NULL comment '所属合作企业',
    establish_date       date         NOT NULL comment '基地成立年月，格式：YYYYMMDD，如：2006-04-12',
    team_status          VARCHAR(255) NOT NULL comment '合作状态',
    protocol_state       varchar(128) null comment '合作协议签署状态',
    team_began_date      date         null comment '合作开始时间',
    team_end_date        date         null comment '合作结束时间'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='校外实训基地数据表';

-- 实训室维护数据表
DROP TABLE IF EXISTS `training_protuct`;
CREATE TABLE `training_protuct`
(
    id                          bigint       not null comment '主键编号' primary key,
    create_user_id              bigint       null comment '创建人',
    create_date                 datetime     null comment '创建时间',
    modify_user_id              bigint       null comment '修改人',
    modify_date                 datetime     null comment '修改时间',
    delete_mark                 int          not null comment '删除标记',
    enabled_mark                int          not null comment '有效标志',
    sort_code                   int          null comment '序号',
    `training_room_name`        varchar(255) NOT NULL COMMENT '实训室名称',
    `training_number`           varchar(32)  NOT NULL COMMENT '实训室号',
    `department_id`             bigint       NOT NULL COMMENT '所属机构(xjr_department)',
    `department_name`           varchar(255) NOT NULL COMMENT '所属机构',
    `school_training_base_id`   bigint       NULL COMMENT '所属实训基地id(school_training_base)',
    `school_training_base_name` varchar(255) NULL COMMENT '所属实训基地',
    `training_createdate`       date         NOT NULL COMMENT '实验室建立日期',
    `contact_id`                bigint       NOT NULL COMMENT '负责人(xjr_user)',
    `contact_name`              varchar(255) NOT NULL COMMENT '负责人',
    `works_number`              int          NOT NULL COMMENT '工位数',
    `room_number`               varchar(255) NOT NULL COMMENT '房间号',
    `room_name`                 varchar(255) NOT NULL COMMENT '房间名称',
    `company_coop_id`           bigint       NULL COMMENT '实训室合作企业id(company_coop)',
    `company_coop_name`         varchar(255) NULL COMMENT '实验室合作企业'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='实训室维护数据表';

-- 实训项目维护
DROP TABLE IF EXISTS `training_project_safe`;
create table training_project_safe
(
    id                     bigint       not null comment '主键编号'
        primary key,
    create_user_id         bigint       null comment '创建人',
    create_date            datetime     null comment '创建时间',
    modify_user_id         bigint       null comment '修改人',
    modify_date            datetime     null comment '修改时间',
    delete_mark            int          not null comment '删除标记',
    enabled_mark           int          not null comment '有效标志',
    sort_code              int          null comment '序号',
    training_name          varchar(255) not null comment '实训项目名称',
    training_code          varchar(255) not null comment '实训项目编号',
    base_major_id          bigint       not null comment '归口专业id(base_major)',
    base_major_name        varchar(255) not null comment '归口专业名称',
    dummy_pro              varchar(25)  not null comment '是否虚拟仿真实训项目',
    skill_request          varchar(255) NULL comment '技能要求',
    assorted_resource_name varchar(512) NULL comment '配套实训资源名称',
    model_count            int          NULL comment '模块数量',
    assess_manner          varchar(255) NULL comment '考核方式',
    out_service            varchar(25)  not null comment '是否对外服务',
    student_class_count    double       NULL comment '学生实训课时数',
    study_year             varchar(255) not null comment '学年（度）',
    base_semester_id       bigint       not null comment '学期id(base_semester)',
    base_semester_name     varchar(255) not null comment '学期名称',
    training_address       varchar(255) NULL comment '实训地点',
    training_protuct_id    bigint       NULL comment '校内实训室id',
    training_protuct_name  varchar(255) NULL comment '校内实训室'
) comment '实训项目维护';

-- 实训教学过程
DROP TABLE IF EXISTS `practical_tt_records`;
CREATE TABLE `practical_tt_records`
(
    `id`                                bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`                    bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`                       datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`                    bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`                       datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`                       int          NOT NULL COMMENT '删除标记',
    `enabled_mark`                      int          NOT NULL COMMENT '有效标志',
    `sort_code`                         int          NULL DEFAULT NULL COMMENT '序号',
    `student_name`                      varchar(255) NOT NULL COMMENT '学生姓名',
    `student_id`                        bigint       NULL DEFAULT NULL COMMENT '学号',
    `name_project`                      varchar(255) NOT NULL COMMENT '实训项目名称',
    base_major_id                       bigint       not null comment '所属专业id(base_major)',
    base_major_name                     varchar(255) not null comment '所属专业名称',
    `supporting_training_software_name` text NOT NULL COMMENT '配套实训软件名称',
    `use_date`                          date         NOT NULL COMMENT '使用日期',
    `duration_training`                 double       NOT NULL COMMENT '学生实训时长',
    `evaluation_results`                varchar(255) NOT NULL COMMENT '实训结果综合评价',
    PRIMARY KEY (`id`) USING BTREE
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='实训教学过程';








