-- 党员干部学习
DROP TABLE IF EXISTS `party_cadre_study`;
CREATE TABLE `party_cadre_study`
(
    id                        BIGINT       NOT NULL COMMENT '主键编号'
        PRIMARY KEY,
    create_user_id            BIGINT       NULL COMMENT '创建人',
    create_date               DATETIME     NULL COMMENT '创建时间',
    modify_user_id            BIGINT       NULL COMMENT '修改人',
    modify_date               DATETIME     NULL COMMENT '修改时间',
    delete_mark               INT          NOT NULL COMMENT '删除标记',
    enabled_mark              INT          NOT NULL COMMENT '有效标志',
    sort_code                 INT          NULL COMMENT '序号',

    party_tissue_id           BIGINT       NOT NULL COMMENT '党组织主键id(party_tissue_manager)',
    party_tissue_name         VARCHAR(256) NOT NULL COMMENT '党组织名称',
    party_tissue_code         VARCHAR(256) NOT NULL COMMENT '党组织编号',

    party_train_way_carrier   VARCHAR(500) NOT NULL COMMENT '党员干部学习培训主要途径和载体',
    party_train_way_content   VARCHAR(500) NOT NULL COMMENT '党员干部学习培训内容',

    participant_teacher_name  Longtext     null comment '活动参与人教师姓名',
    participant_student_name  Longtext     null comment '活动参与人学生姓名',

    start_time                DATE         NOT NULL COMMENT '活动开始日期',
    end_time                  DATE         NOT NULL COMMENT '活动结束日期',

    participate_teacher_count INT          NULL COMMENT '参与教师数',
    participate_student_count INT          NULL COMMENT '参与学生数'
) ENGINE = INNODB
  DEFAULT CHARSET = utf8mb4
    COMMENT '党员干部学习';

# 数据字典
# par_part 党员干部学习培训内容
# 值  名称               编号
# 1 党的理论              par_part1
# 2 相关政策文件            par_part2
# 3 党规党纪及国家法律法规   par_part3
# 4 教育教学业务            par_part4
# 5 学校管理              par_part5
# 6 文化建设              par_part6
# 7 经济方面              par_part7
# 8 互联网与新媒体方面     par_part8
# 9 心理健康              par_part9
# 10 师德师风             par_part10
# 99 其他                 par_part99
#
# party_way 党员干部学习培训主要途径和载体
# 值  名称               编号
# 1  实践活动             party_way1
# 2  会议               party_way2
# 3  讲座报告             party_way3
# 4  学校定期集中学习         party_way4
# 5  支部定期集中学习         party_way5
# 6   微信 QQ               party_way6
# 7  移动端培训平台          party_way7
# 8  分发图书资料           party_way8
# 9  个人学习             party_way9
# 10 在线教育网站           party_way10
# 11 参观学习             party_way11
# 99 其他                   party_way99
-- 党课管理数据表
DROP TABLE IF EXISTS party_class_manage;
CREATE TABLE party_class_manage
(
    id                        BIGINT       NOT NULL COMMENT '主键编号' PRIMARY KEY,
    create_user_id            BIGINT       NULL COMMENT '创建人',
    create_date               DATETIME     NULL COMMENT '创建时间',
    modify_user_id            BIGINT       NULL COMMENT '修改人',
    modify_date               DATETIME     NULL COMMENT '修改时间',
    delete_mark               INT          NOT NULL COMMENT '删除标记',
    enabled_mark              INT          NOT NULL COMMENT '有效标志',
    sort_code                 INT          NULL COMMENT '序号',

    party_tissue_name         VARCHAR(256) NOT NULL COMMENT '党组织名称',
    party_tissue_code         VARCHAR(256) NOT NULL COMMENT '党组织编号',
    party_tissue_id           BIGINT       NOT NULL COMMENT '党组织主键id(party_tissue_manager)',
    participant_teacher_name  Longtext     null comment '活动参与人教师姓名',
    participant_student_name  Longtext     null comment '活动参与人学生姓名',

    activity_type             varchar(256) null comment '活动形式',
    speaker_name              varchar(256) null comment '党课主讲人',

    start_time                DATE         NOT NULL COMMENT '活动开始时间',
    end_time                  DATE         NOT NULL COMMENT '活动结束时间',

    activities_content        TEXT         NULL COMMENT '活动内容',

    participate_teacher_count INT          NULL COMMENT '参与教师数',
    participate_student_count INT          NULL COMMENT '参与学生数'

) ENGINE = INNODB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='党课管理数据表';

-- 党建活动
DROP TABLE IF EXISTS `party_build_event`;
CREATE TABLE party_build_event
(
    id                       BIGINT       NOT NULL COMMENT '主键编号'
        PRIMARY KEY,
    create_user_id           BIGINT       NULL COMMENT '创建人',
    create_date              DATETIME     NULL COMMENT '创建时间',
    modify_user_id           BIGINT       NULL COMMENT '修改人',
    modify_date              DATETIME     NULL COMMENT '修改时间',
    delete_mark              INT          NOT NULL COMMENT '删除标记',
    enabled_mark             INT          NOT NULL COMMENT '有效标志',
    sort_code                INT          NULL COMMENT '序号',

    event_type               VARCHAR(255) NOT NULL COMMENT '活动类型',
    party_tissue_id          BIGINT       NOT NULL COMMENT '党组织主键id(party_tissue_manager)',
    party_tissue_name        VARCHAR(256) NOT NULL COMMENT '党组织名称',
    party_tissue_code        VARCHAR(256) NOT NULL COMMENT '党组织编号',
    participant_teacher_name Longtext     null comment '活动参与人教师姓名',
    participant_student_name Longtext     null comment '活动参与人学生姓名',

    event_address            VARCHAR(255) NULL COMMENT '活动地点',
    event_content            TEXT         NULL COMMENT '活动内容',
    event_start_time         DATE         NOT NULL COMMENT '活动开始时间',
    event_end_time           DATE         NOT NULL COMMENT '活动结束时间',
    attend_teacher_num       INT          NULL COMMENT '参于教师数',
    attend_student_num       INT          NULL COMMENT '参于学生数'
) ENGINE = INNODB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='党建活动';

