-- 教研培训
DROP TABLE IF EXISTS `wf_teacher_training`;
CREATE TABLE `wf_teacher_training`
(
    id               bigint       not null comment '主键编号'
        primary key,
    create_user_id   bigint       null comment '创建人',
    create_date      datetime     null comment '创建时间',
    modify_user_id   bigint       null comment '修改人',
    modify_date      datetime     null comment '修改时间',
    delete_mark      int          not null comment '删除标记',
    enabled_mark     int          not null comment '有效标志',
    sort_code        int          null comment '序号',
    training_name    varchar(256) null comment '培训活动名称',
    training_topic   text         null comment '培训活动主题',
    person_in_charge varchar(256) null comment '培训负责人',
    start_time       datetime     null comment '培训开始时间',
    end_time         datetime     null comment '培训结束时间',
    training_address varchar(512) null comment '培训地点',
    `remark`         VARCHAR(1000)         DEFAULT NULL COMMENT '备注',
    `status`         INT          NOT NULL DEFAULT '0' COMMENT '状态（1:结束 0:未结束）'
)
    comment 'wf_teacher_training';

-- 教研培训参与培训人员
DROP TABLE IF EXISTS `wf_teacher_training_participant`;
CREATE TABLE `wf_teacher_training_participant`
(
    id                     bigint       not null comment '主键编号'
        primary key,
    create_user_id         bigint       null comment '创建人',
    create_date            datetime     null comment '创建时间',
    modify_user_id         bigint       null comment '修改人',
    modify_date            datetime     null comment '修改时间',
    delete_mark            int          not null comment '删除标记',
    enabled_mark           int          not null comment '有效标志',
    sort_code              int          null comment '序号',
    wf_teacher_training_id bigint       null comment '教研培训主键id（wf_teacher_training）',
    participant_id         bigint       null comment '参与培训人id',
    participant_username   varchar(256) null comment '参与培训人oa',
    participant_name       varchar(256) null comment '参与培训人姓名'
)
    comment '教研培训参与培训人员';

-- 校内实训基地数据项
DROP TABLE IF EXISTS `school_training_base`;
CREATE TABLE school_training_base
(
    id                     bigint        not null comment '主键编号'
        primary key,
    create_user_id         bigint        null comment '创建人',
    create_date            datetime      null comment '创建时间',
    modify_user_id         bigint        null comment '修改人',
    modify_date            datetime      null comment '修改时间',
    delete_mark            int           not null comment '删除标记',
    enabled_mark           int           not null comment '有效标志',
    sort_code              int           null comment '序号',
    training_base_name     VARCHAR(255)  NOT NULL COMMENT '实训基地名称',
    training_base_number   VARCHAR(255)  null DEFAULT '0' COMMENT '实训基地编号',
    training_rooms_number  INT           NOT NULL COMMENT '实训室数，数字标识，例：22',
    training_project_total INT           NOT NULL COMMENT '实训项目总数，数字标识，例：33',
    base_category          VARCHAR(255)  NOT NULL COMMENT '基地类别代码（SXJDLBDM）',
    approve_date           date          NOT NULL COMMENT '批准日期，格式：YYYYMMDD，如：2006-04-12',
    setup_year             VARCHAR(10)   NOT NULL COMMENT '成立年度，格式：YYYY，如：2006',
    base_major_id          bigint        null DEFAULT null COMMENT '面向专业',
    base_major_name        VARCHAR(256)  null DEFAULT '' COMMENT '专业名字',
    support_department     VARCHAR(255)  null DEFAULT '' COMMENT '被列为实训基地项目支持部门',
    build_area             DOUBLE(10, 2) null DEFAULT 0 COMMENT '建筑面积',
    instrument_total       INT           null DEFAULT 0 COMMENT '仪器设备总数，数字标识，例：33',
    station_number         INT           null DEFAULT 0 COMMENT '实践教学工位数，数字标识，例：33',
    manager_number_allied  INT           null DEFAULT 0 COMMENT '管理人员数（专职）',
    manager_number_part    INT           null DEFAULT 0 COMMENT '管理人员数（兼职）'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='校内实训基地数据表';

-- 学生证书表
DROP TABLE IF EXISTS `student_cert_table`;
CREATE TABLE student_cert_table
(
    id                 bigint       not null comment '主键编号'
        primary key,
    create_user_id     bigint       null comment '创建人',
    create_date        datetime     null comment '创建时间',
    modify_user_id     bigint       null comment '修改人',
    modify_date        datetime     null comment '修改时间',
    delete_mark        int          not null comment '删除标记',
    enabled_mark       int          not null comment '有效标志',
    sort_code          int          null comment '序号',
    student_id         varchar(64)  NOT NULL COMMENT '学号',
    student_name       varchar(64)  not null comment '学生姓名',
    student_gender     varchar(32)  NOT NULL COMMENT '性别',
    class_id           bigint       not null comment '学生班级id',
    class_name         varchar(64)  not null comment '学生班级',
    teacher_id         bigint       NOT NULL COMMENT '教师id',
    teacher_name       varchar(64)  NOT NULL COMMENT '教师姓名',
    cert_name          varchar(512) NOT NULL COMMENT '证书名称',
    cert_id            varchar(256) NOT NULL COMMENT '证书编号',
    cert_grade         varchar(64)  NULL COMMENT '证书等级',
    cert_organizations varchar(255) NOT NULL COMMENT '证书发放机构',
    cert_date          date         NULL COMMENT '证书发放日期',
    cert_electronic    varchar(64)  NOT NULL COMMENT '是否支持电子证书(0：否，1：是)',
    remark             varchar(1024) DEFAULT NULL COMMENT '备注',
    file_id            longtext     NOT NULL COMMENT '附件',
    name               VARCHAR(30)  NOT NULL COMMENT '学生姓名',
    status             int          NOT NULL COMMENT '状态（1:结束 0:未结束）'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='学生证书表';

-- 合作企业表
DROP TABLE IF EXISTS `company_coop`;
CREATE TABLE `company_coop`
(
    id                 bigint       not null comment '主键编号'
        primary key,
    create_user_id     bigint       null comment '创建人',
    create_date        datetime     null comment '创建时间',
    modify_user_id     bigint       null comment '修改人',
    modify_date        datetime     null comment '修改时间',
    delete_mark        int          not null comment '删除标记',
    enabled_mark       int          not null comment '有效标志',
    sort_code          int          null comment '序号',
    `company_name`     varchar(255) NOT NULL COMMENT '企业名称',
    `company_type`     varchar(64)  NOT NULL COMMENT '企业性质(企业性质字典编号)',
    `company_size`     varchar(64)  NOT NULL COMMENT '企业规模（企业规模字典标号）',
    `company_person`   varchar(255) NOT NULL COMMENT '法人代表',
    `company_position` varchar(255) NOT NULL COMMENT '企业地址',
    `contact`          varchar(255) NOT NULL COMMENT '联系人',
    `phone`            varchar(32)  NOT NULL COMMENT '联系电话',
    `partner_type`     varchar(64)  NOT NULL COMMENT '合作状态（状态类型字典编号）',
    `sector`           varchar(64)  NOT NULL COMMENT '所属行业(行业类型字典编号)',
    `industry`         varchar(64)  NOT NULL COMMENT '所属产业（产业类型字典编号）',
    `company_email`    varchar(255) NULL DEFAULT NULL COMMENT '企业邮箱',
    `contact_email`    varchar(255) NULL DEFAULT NULL COMMENT '联系人邮箱',
    `company_fax`      varchar(255) NULL DEFAULT NULL COMMENT '企业传真',
    `company_register` varchar(255) NULL DEFAULT NULL COMMENT '企业注册号',
    `company_website`  varchar(512) NULL DEFAULT NULL COMMENT '企业网址'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='合作企业表';

-- 奖助学金申请表
DROP TABLE IF EXISTS `scholarship_bursary_applications`;
CREATE TABLE `scholarship_bursary_applications`
(
    `id`                                   bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`                       bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`                          datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`                       bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`                          datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`                          int          NOT NULL COMMENT '删除标记',
    `enabled_mark`                         int          NOT NULL COMMENT '有效标志',
    `sort_code`                            int          NULL DEFAULT NULL COMMENT '序号',
    `class_name`                           varchar(256) NOT NULL COMMENT '班级',
    `teacher_id`                           bigint       NOT NULL COMMENT '班主任id',
    `teacher_name`                         varchar(256) NOT NULL COMMENT '班主任姓名',
    `student_name`                         varchar(64)  not null comment '学生姓名',
    `student_gender`                       varchar(32)  NOT NULL COMMENT '学生性别',
    `identification_number`                varchar(64)  NOT NULL COMMENT '身份证号',
    `base_semester`                        varchar(256) NOT NULL COMMENT '学期',
    `base_student_scholarship_category_id` varchar(256) NOT NULL COMMENT '申请奖助学金名称',
    `base_student_scholarship_category`    varchar(256) NOT NULL COMMENT '申请奖助学金名称',
    `personal_circumstances`               longtext     NOT NULL COMMENT '描述个人情况',
    `application_reason`                   longtext     NOT NULL COMMENT '阐述申请理由',
    `bank_name`                            varchar(256) NOT NULL COMMENT '收款银行名称',
    `bank_card_number`                     varchar(256) NOT NULL COMMENT '银行卡号',
    `bank_account_name`                    varchar(256) NOT NULL COMMENT '账户名称',
    `file_id`                              longtext     NULL COMMENT '佐证材料',
    `status`                               int          NULL DEFAULT 0 COMMENT '状态（1:结束 0:未结束）',
    PRIMARY KEY (`id`) USING BTREE
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='奖助学金申请表';

DROP TABLE IF EXISTS base_digital_resources;
CREATE TABLE IF NOT EXISTS base_digital_resources
(
    id                bigint       not null comment '主键编号'
        primary key,
    create_user_id    bigint       null comment '创建人',
    create_date       datetime     null comment '创建时间',
    modify_user_id    bigint       null comment '修改人',
    modify_date       datetime     null comment '修改时间',
    delete_mark       int          not null comment '删除标记',
    enabled_mark      int          not null comment '有效标志',
    sort_code         int          null comment '序号',

    resourse_name     varchar(300) not null comment '资源名称',
    resourse_code     varchar(100) not null comment '资源编号',
    major_code        varchar(50)  null comment '所属专业号',
    teacher_code      varchar(50)  null comment '资源负责人号',
    resourse_level    varchar(2)   null comment '资源级别码',
    resourse_type     varchar(15)  null comment '资源类型码',
    teaching_platform varchar(2)   null comment '国家智慧教学平台资源类型码',
    resourse_create   datetime     null comment '资源创建时间',
    resourse_update   datetime     null comment '资源更新时间',
    course_code       varchar(20)  null comment '关联课程号',
    major_name        VARCHAR(30) NULL COMMENT '面向专业名'
) ENGINE = innodb
  DEFAULT CHARSET = utf8mb4 COMMENT '数字资源';


  -- 校外实训基地数据项
DROP TABLE IF EXISTS `outside_training_base`;
CREATE TABLE outside_training_base
(
    id                   bigint       not null comment '主键编号' primary key,
    create_user_id       bigint       null comment '创建人',
    create_date          datetime     null comment '创建时间',
    modify_user_id       bigint       null comment '修改人',
    modify_date          datetime     null comment '修改时间',
    delete_mark          int          not null comment '删除标记',
    enabled_mark         int          not null comment '有效标志',
    sort_code            int          null comment '序号',
    training_base_name   VARCHAR(255) NOT NULL comment '实训基地名称',
    training_base_number VARCHAR(255) null comment '实训基地编号',
    company_coop_id      bigint       NOT NULL comment '所属合作企业(company_coop)',
    company_coop_name    varchar(300) NOT NULL comment '所属合作企业',
    establish_date       date         NOT NULL comment '基地成立年月，格式：YYYYMMDD，如：2006-04-12',
    team_status          VARCHAR(255) NOT NULL comment '合作状态',
    protocol_state       varchar(128) null comment '合作协议签署状态',
    team_began_date      date         null comment '合作开始时间',
    team_end_date        date         null comment '合作结束时间'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='校外实训基地数据表';

-- 实训室维护数据表
DROP TABLE IF EXISTS `training_protuct`;
CREATE TABLE `training_protuct`
(
    id                          bigint       not null comment '主键编号' primary key,
    create_user_id              bigint       null comment '创建人',
    create_date                 datetime     null comment '创建时间',
    modify_user_id              bigint       null comment '修改人',
    modify_date                 datetime     null comment '修改时间',
    delete_mark                 int          not null comment '删除标记',
    enabled_mark                int          not null comment '有效标志',
    sort_code                   int          null comment '序号',
    `training_room_name`        varchar(255) NOT NULL COMMENT '实训室名称',
    `training_number`           varchar(32)  NOT NULL COMMENT '实训室号',
    `department_id`             bigint       NOT NULL COMMENT '所属机构(xjr_department)',
    `department_name`           varchar(255) NOT NULL COMMENT '所属机构',
    `school_training_base_id`   bigint       NULL COMMENT '所属实训基地id(school_training_base)',
    `school_training_base_name` varchar(255) NULL COMMENT '所属实训基地',
    `training_createdate`       date         NOT NULL COMMENT '实验室建立日期',
    `contact_id`                bigint       NOT NULL COMMENT '负责人(xjr_user)',
    `contact_name`              varchar(255) NOT NULL COMMENT '负责人',
    `works_number`              int          NOT NULL COMMENT '工位数',
    `room_number`               varchar(255) NOT NULL COMMENT '房间号',
    `room_name`                 varchar(255) NOT NULL COMMENT '房间名称',
    `company_coop_id`           bigint       NULL COMMENT '实训室合作企业id(company_coop)',
    `company_coop_name`         varchar(255) NULL COMMENT '实验室合作企业',
    `contact_oa`                varchar(20) NOT NULL COMMENT '负责人oa'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='实训室维护数据表';

-- 实训项目维护
DROP TABLE IF EXISTS `training_project_safe`;
create table training_project_safe
(
    id                     bigint       not null comment '主键编号'
        primary key,
    create_user_id         bigint       null comment '创建人',
    create_date            datetime     null comment '创建时间',
    modify_user_id         bigint       null comment '修改人',
    modify_date            datetime     null comment '修改时间',
    delete_mark            int          not null comment '删除标记',
    enabled_mark           int          not null comment '有效标志',
    sort_code              int          null comment '序号',
    training_name          varchar(255) not null comment '实训项目名称',
    training_code          varchar(255) not null comment '实训项目编号',
    base_major_id          bigint       not null comment '归口专业id(base_major)',
    base_major_name        varchar(255) not null comment '归口专业名称',
    dummy_pro              varchar(25)  not null comment '是否虚拟仿真实训项目',
    skill_request          varchar(255) NULL comment '技能要求',
    assorted_resource_name varchar(512) NULL comment '配套实训资源名称',
    model_count            int          NULL comment '模块数量',
    assess_manner          varchar(255) NULL comment '考核方式',
    out_service            varchar(25)  not null comment '是否对外服务',
    student_class_count    double       NULL comment '学生实训课时数',
    study_year             varchar(255) not null comment '学年（度）',
    base_semester_id       bigint       not null comment '学期id(base_semester)',
    base_semester_name     varchar(255) not null comment '学期名称',
    training_address       varchar(255) NULL comment '实训地点',
    training_protuct_id    bigint       NULL comment '校内实训室id',
    training_protuct_name  varchar(255) NULL comment '校内实训室'
) comment '实训项目维护';

-- 实训教学过程
DROP TABLE IF EXISTS `practical_tt_records`;
CREATE TABLE `practical_tt_records`
(
    `id`                                bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`                    bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`                       datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`                    bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`                       datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`                       int          NOT NULL COMMENT '删除标记',
    `enabled_mark`                      int          NOT NULL COMMENT '有效标志',
    `sort_code`                         int          NULL DEFAULT NULL COMMENT '序号',
    `student_name`                      varchar(255) NOT NULL COMMENT '学生姓名',
    `student_id`                        bigint       NULL DEFAULT NULL COMMENT '学号',
    `name_project`                      varchar(255) NOT NULL COMMENT '实训项目名称',
    base_major_id                       bigint       not null comment '所属专业id(base_major)',
    base_major_name                     varchar(255) not null comment '所属专业名称',
    `supporting_training_software_name` text NOT NULL COMMENT '配套实训软件名称',
    `use_date`                          date         NOT NULL COMMENT '使用日期',
    `duration_training`                 double       NOT NULL COMMENT '学生实训时长',
    `evaluation_results`                varchar(255) NOT NULL COMMENT '实训结果综合评价',
    PRIMARY KEY (`id`) USING BTREE
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='实训教学过程';



  -- 学生消费数据
DROP TABLE IF EXISTS `student_consume_date`;
CREATE TABLE student_consume_date
(
    id                bigint         NOT NULL COMMENT '主键编号' PRIMARY KEY,
    create_user_id    bigint         NULL COMMENT '创建人',
    create_date       datetime       NULL COMMENT '创建时间',
    modify_user_id    bigint         NULL COMMENT '修改人',
    modify_date       datetime       NULL COMMENT '修改时间',
    delete_mark       int            NOT NULL COMMENT '删除标记',
    enabled_mark      int            NOT NULL COMMENT '有效标志',
    sort_code         int            NULL COMMENT '序号',

    class_id          bigint         NOT NULL COMMENT '班级id',
    class_name        VARCHAR(256)   NOT NULL COMMENT '班级名称',
    student_id        bigint         NOT NULL COMMENT '学生id',
    student_name      VARCHAR(256)   NULL COMMENT '学生姓名',
    student_user_name VARCHAR(256)   NULL COMMENT '学生学号',
    consume_time      datetime       NOT NULL COMMENT '消费时间',
    consume_money     DECIMAL(10, 2) NOT NULL COMMENT '消费金额',
    consume_type      VARCHAR(16)    NOT NULL COMMENT '消费类型',
    grade_name        VARCHAR(256)   NULL COMMENT '年级名称',
    grade_id          VARCHAR(256)   NULL COMMENT '年级id',
    major_name        VARCHAR(256)   NULL COMMENT '专业名称',
    major_id          VARCHAR(256)   NULL COMMENT '专业id'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='学生消费数据';

DROP TABLE IF EXISTS base_course_subject;
CREATE TABLE `base_course_subject`
(
    `id`             bigint      NOT NULL COMMENT '主键',
    `create_user_id` bigint       DEFAULT NULL COMMENT '创建人',
    `create_date`    date         DEFAULT NULL COMMENT '创建时间',
    `modify_user_id` bigint       DEFAULT NULL COMMENT '修改人',
    `modify_date`    date         DEFAULT NULL COMMENT '修改时间',
    `delete_mark`    int         NOT NULL COMMENT '删除标记',
    `enabled_mark`   int         NOT NULL COMMENT '有效标志',
    `name`           varchar(30) NOT NULL COMMENT '课程学科名称',
    `code`           varchar(30)  DEFAULT NULL COMMENT '课程学科代码',
    `remark`         varchar(250) DEFAULT NULL COMMENT '备注',
    `base_label_id`  bigint       DEFAULT NULL COMMENT '标签id(base_label)',
    PRIMARY KEY (`id`) USING BTREE
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
  COLLATE = utf8mb4_0900_ai_ci COMMENT ='课程学科';

DROP TABLE IF EXISTS course_subject_detail;
CREATE TABLE IF NOT EXISTS course_subject_detail
(
    id                     bigint       NOT NULL COMMENT '主键编号'
        PRIMARY KEY,
    create_user_id         bigint       NULL COMMENT '创建人',
    create_date            datetime     NULL COMMENT '创建时间',
    modify_user_id         bigint       NULL COMMENT '修改人',
    modify_date            datetime     NULL COMMENT '修改时间',
    delete_mark            int          NOT NULL COMMENT '删除标记',
    enabled_mark           int          NOT NULL COMMENT '有效标志',
    sort_code              int          NULL COMMENT '序号',

    base_course_subject_id bigint       NOT NULL COMMENT '课程学科管理主表id',
    course_category        varchar(16)  NOT NULL COMMENT '课程类别',
    course_nature          varchar(16)  NOT NULL COMMENT '课程性质',
    course_attribute       varchar(16)  NOT NULL COMMENT '课程属性',
    course_type            varchar(16)  NOT NULL COMMENT '课程分类',
    subject_quality        varchar(16)  NOT NULL COMMENT '学科类别',
    core_course            varchar(16)  NOT NULL COMMENT '是否专业核心课程',
    theory_time            int          NOT NULL COMMENT '理论教学时数',
    practice_time          int          NOT NULL COMMENT '实践教学时数',
    online_course          varchar(20)   NOT NULL COMMENT '是否有线上课程',
    course_url             longtext     NULL COMMENT '线上课程网址',
    ideology_demo          varchar(20)   NOT NULL COMMENT '是否思政示范课程',
    course_resource        varchar(10)  NULL DEFAULT '' COMMENT '课堂教学设计主要选用课程资源',
    emulation_time         int          NOT NULL COMMENT '虚拟仿真实训课时数',
    emulation_name         varchar(256) NULL DEFAULT '' COMMENT '主要开设虚拟仿真实训项目名称',
    experiment_name        varchar(256) NULL DEFAULT '' COMMENT '主要开设实验项目名称',
    training_name          varchar(256) NULL DEFAULT '' COMMENT '主要开设实训项目名称',
    internship_name        varchar(256) NULL DEFAULT '' COMMENT '主要开设实习项目名称'
) ENGINE = innodb
  DEFAULT CHARSET = utf8mb4 COMMENT '课程学科管理详情表';


DROP TABLE IF EXISTS base_student_scholarship_applicant;
create table if not exists base_student_scholarship_applicant
(
    id                                      bigint       NOT NULL COMMENT '主键编号'
        PRIMARY KEY,
    create_user_id                          bigint       NULL COMMENT '创建人',
    create_date                             datetime     NULL COMMENT '创建时间',
    modify_user_id                          bigint       NULL COMMENT '修改人',
    modify_date                             datetime     NULL COMMENT '修改时间',
    delete_mark                             int          NOT NULL COMMENT '删除标记',
    enabled_mark                            int          NOT NULL COMMENT '有效标志',
    sort_code                               int          NULL COMMENT '序号',

    base_student_scholarship_category_id    bigint       NULL COMMENT '奖学金类别id',
    scholarship_level                       int          DEFAULT 0       NOT NULL COMMENT '奖学金等级0（无等级）',
    amount                                  double       DEFAULT 0       NOT NULL COMMENT '金额',
    applicant_user_id                       bigint       NULL COMMENT '申请人',
    base_semester_id                        bigint       NULL COMMENT '学期id',
    grade_name                              varchar(200) NULL COMMENT '年级名称',
    class_name                              varchar(200) NULL COMMENT '班级名称',
    name                                    varchar(200) NULL COMMENT '姓名',
    student_id                              varchar(200) NULL COMMENT '学号',
    gender_name                             varchar(200) NULL COMMENT '性别',
    enroll_type_cn                          varchar(200) NULL COMMENT '招生类型名称',
    major_name                              varchar(200) NULL COMMENT '在读专业',
    award_date                              date         NULL COMMENT '获奖日期',
    bank_type                               varchar(20)  NULL COMMENT '收款银行',
    bank_no                                 varchar(200) NULL COMMENT '银行卡号',
    file_id                                 bigint       NULL COMMENT '文件id', 
    status                                  int          DEFAULT 0       NOT NULL COMMENT '状态(1:结束 0:未结束)',
    release_status                          int          DEFAULT 0       NOT NULL COMMENT '发放状态(1:已发放 0:未发放)',
    bank_user_name                          varchar(50)  null COMMENT '银行账户名称',     
    review_status                           int          DEFAULT 0       NULL COMMENT '评审状态',
    identification_number                   varchar(256) NOT NULL COMMENT '身份证号',
    base_student_scholarship_name           varchar(256) NOT NULL COMMENT '申请奖助学金名称',
    personal_circumstances                  longtext     NOT NULL COMMENT '描述个人情况',
    application_reason                      longtext     NOT NULL COMMENT '阐述申请理由',
    teacher_id                              BIGINT       NOT NULL COMMENT '班主任id'
) ENGINE = innodb
 DEFAULT CHARSET = utf8mb4 COMMENT '奖学金申请';


-- 思政德育活动
DROP TABLE IF EXISTS `moral_event`;
CREATE TABLE moral_event
(
    id                 bigint       not null comment '主键编号'
        primary key,
    create_user_id     bigint       null comment '创建人',
    create_date        datetime     null comment '创建时间',
    modify_user_id     bigint       null comment '修改人',
    modify_date        datetime     null comment '修改时间',
    delete_mark        int          not null comment '删除标记',
    enabled_mark       int          not null comment '有效标志',
    sort_code          int          null comment '序号',

    moral_event_name   varchar(255) not null comment '德育活动名称',
    belong_theme       varchar(255) null comment '所属主题',
    event_plate        varchar(255) null comment '活动板块',
    event_type         varchar(255) not null comment '活动类型',
    event_theme        varchar(255) null comment '活动主题',
    event_leader       varchar(255) null comment '活动负责人',
    host_unit          varchar(255) not null comment '主办单位',
    host_unit_lever    varchar(255) not null comment '主办单位级别',
    event_address      varchar(255) null comment '活动地点',
    event_content      text         null comment '活动内容',
    event_start_time   date         not null comment '活动开始时间',
    event_end_time     date         not null comment '活动结束时间',
    attend_class       longtext     not null comment '参于班级',
    attend_class_num   int          null comment '参于班级数',
    attend_teacher     longtext     not null comment '参于教师',
    attend_teacher_num int          null comment '参于教师数',
    attend_student_num int          null comment '参于学生数'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='思政德育活动';

-- 教师进修
DROP TABLE IF EXISTS `teacher_trainee`;
CREATE TABLE teacher_trainee
(
    id                   bigint       not null comment '主键编号'
        primary key,
    create_user_id       bigint       null comment '创建人',
    create_date          datetime     null comment '创建时间',
    modify_user_id       bigint       null comment '修改人',
    modify_date          datetime     null comment '修改时间',
    delete_mark          int          not null comment '删除标记',
    enabled_mark         int          not null comment '有效标志',
    sort_code            int          null comment '序号',

    teacher_id           bigint       not null comment '进修教师id',
    teacher_name         varchar(255) not null comment '教师姓名',
    teacher_user_name    varchar(255) not null comment '教师工号or教师oa',
    trainee_name         varchar(255) not null comment '进修培训名称',
    trainee_number       varchar(255) not null comment '进修培训编号',
    trainee_theme        varchar(255) null comment '进修培训主题',
    trainee_content      varchar(255) null comment '进修培训内容',
    trainee_main_speaker varchar(255) null comment '进修培训主讲人',
    start_time           date         not null comment '进修培训开始日期',
    end_time             date         not null comment '进修培训截止日期',
    trainee_place        varchar(255) not null comment '进修培训地点',
    out_date             date         null comment '出国（境）日期',
    out_purpose          varchar(255) null comment '出国（境）目的',
    out_unit             varchar(255) null comment '派出单位',
    approval_unit        varchar(255) null comment '审批单位',
    approval_date        date         null comment '审批日期',
    return_date          date         null comment '应回国日期'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='教师进修';

-- 课程互动记录表
DROP TABLE IF EXISTS `classroom_interaction_record`;
CREATE TABLE `classroom_interaction_record`
(
    `id`                            bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`                bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`                   datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`                bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`                   datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`                   int          NOT NULL COMMENT '删除标记',
    `enabled_mark`                  int          NOT NULL COMMENT '有效标志',
    `sort_code`                     int          NULL COMMENT '序号',


    `class_grade_id`                varchar(255) NOT NULL COMMENT '班级id',
    `class_grade_name`              varchar(255) NOT NULL COMMENT '班级',
    `major_information_id`          varchar(255) NOT NULL COMMENT '专业信息id',
    `major_information`             varchar(255) NOT NULL COMMENT '专业信息',
    `grade_information_id`          varchar(255) NOT NULL COMMENT '年级信息id',
    `grade_information`             varchar(255) NOT NULL COMMENT '年级信息',
    `week_time`                     varchar(255) NOT NULL COMMENT '星期',
    `week_number`                   int          NOT NULL COMMENT '周次',
    `section_number`                varchar(255) NOT NULL COMMENT '节次',
    `course_schedule_number`        varchar(255) NOT NULL COMMENT '排课号',
    `course_name`                   varchar(255) NOT NULL COMMENT '课程名称',
    `course_nunber`                 varchar(255) NOT NULL COMMENT '课程号',
    `class_data`                    date         NOT NULL COMMENT '上课日期',
    `class_time`                    varchar(255) NOT NULL COMMENT '上课时间',
    `number_interactions_initiated` bigint       NOT NULL COMMENT '互动发起次数',
    `number_student_participations` bigint       NOT NULL COMMENT '学生参与总次数',
    `attendance_rate`               varchar(255) NOT NULL COMMENT '出勤率',
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT = '课程互动记录';

-- 综合成绩
DROP TABLE IF EXISTS `combine_cert`;
CREATE TABLE `combine_cert`
(
    id                             bigint         not null comment '主键编号'
        primary key,
    create_user_id                 bigint         null comment '创建人',
    create_date                    datetime       null comment '创建时间',
    modify_user_id                 bigint         null comment '修改人',
    modify_date                    datetime       null comment '修改时间',
    delete_mark                    int            not null comment '删除标记',
    enabled_mark                   int            not null comment '有效标志',
    sort_code                      int            null comment '序号',

    base_semester                  varchar(256)   NOT NULL COMMENT '学期',
    base_semester_id               bigint         NOT NULL COMMENT '学期id',
    user_id                        bigint         NOT NULL COMMENT '学生id',
    student_id                     varchar(255)   NOT NULL COMMENT '学号',
    student_name                   varchar(255)   not null comment '学生姓名',
    student_gender                 varchar(32)    NOT NULL COMMENT '性别',
    base_grade_name                VARCHAR(256)   not null COMMENT '年级',
    class_id                       bigint         not null comment '班级id',
    class_name                     varchar(64)    not null comment '学生班级',
    base_major_name                VARCHAR(256)   not null COMMENT '专业名称',
    teacher_name                   varchar(64)    NOT NULL COMMENT '教师姓名',

    thought_politics_score         DECIMAL(10, 1) not null comment '思想政治成绩',
    culture_score                  DECIMAL(10, 1) not null comment '文化课成绩',
    expertise_score                DECIMAL(10, 1) not null comment '专业技能成绩',
    student_health_score           DECIMAL(10, 1) not null comment '学生体质健康成绩',
    omprehensive_training          DECIMAL(10, 1) not null comment '综合实训成绩',
    post_exercitation_score        DECIMAL(10, 1) not null comment '岗位实践成绩',
    comprehensive_evaluation_score DECIMAL(10, 1) not null comment '综合评价成绩',
    academic_year                   VARCHAR(9)    NOT null comment '学年',
    semester_code                   VARCHAR(16)   NOT null comment '学期码'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    comment '综合成绩';


-- 教职工考核
DROP TABLE IF EXISTS `faculty_assess`;
CREATE TABLE `faculty_assess`
(
    id                       bigint       not null comment '主键编号'
        primary key,
    create_user_id           bigint       null comment '创建人',
    create_date              datetime     null comment '创建时间',
    modify_user_id           bigint       null comment '修改人',
    modify_date              datetime     null comment '修改时间',
    delete_mark              int          not null comment '删除标记',
    enabled_mark             int          not null comment '有效标志',
    sort_code                int          null comment '序号',

    teacher_name             varchar(255) NOT NULL COMMENT '考核教师',
    assess_date              date         NOT NULL COMMENT '考核日期',
    assess_type              varchar(255) COMMENT '考核类型',
    xjr_department_name      varchar(255) NOT NULL COMMENT '考核部门',
    dept_assess_result       varchar(255) NOT NULL COMMENT '部门考核结果',
    asses_head_assess_result varchar(255) COMMENT '考核负责人',
    school_assess_result     varchar(255) COMMENT '学校考核结果',
    assess_content           varchar(500) COMMENT '考核内容',
    teacher_user_name        varchar(64)  NOT NULL COMMENT '用户名',

    teacher_name_t          VARCHAR(36) NOT NULL COMMENT '考核教师姓名',
    assess_person_name      VARCHAR(36) NOT NULL COMMENT '考核负责人姓名',
    assess_person_username  VARCHAR(20) NOT NULL COMMENT '考核负责人工号',
    assess_unit_number      VARCHAR(10) NOT NULL COMMENT '教职工考核单位号，学校自编',
    `xjr_department_name_t` varchar(60) NOT NULL COMMENT '考核部门'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    comment '教职工考核';

-- 实习报告表
DROP TABLE IF EXISTS `internship_report`;
CREATE TABLE `internship_report`
(
    `id`                        bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`            bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`               datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`            bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`               datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`               int          NOT NULL COMMENT '删除标记',
    `enabled_mark`              int          NOT NULL COMMENT '有效标志',
    `sort_code`                 int          NULL DEFAULT NULL COMMENT '序号',

    `student_name`              varchar(255) NOT NULL COMMENT '姓名',
    `student_id`                bigint       NOT NULL COMMENT '学号',
    `class_name`                varchar(255) NOT NULL COMMENT '班级',
    `teacher_id`                bigint       NOT NULL COMMENT '班主任id',
    `teacher_name`              varchar(255) NOT NULL COMMENT '班主任',
    `program_name`              varchar(255) NOT NULL COMMENT '实习计划名称',
    `internship_unit`           varchar(255) NULL COMMENT '实习单位',
    `internship_positions`      varchar(255) NULL COMMENT '实习岗位',
    `internship_report_content` text         NOT NULL COMMENT '实习报告内容',
    `report_category`           varchar(255) NOT NULL COMMENT '报告类型',
    `status`                    int          NOT NULL COMMENT '状态',
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT = '实习报告表';


-- 教学质量与评价
DROP TABLE IF EXISTS `teaching_quality_evaluation`;
CREATE TABLE `teaching_quality_evaluation`
(
    `id`                bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`    bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`       datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`    bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`       datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`       int          NOT NULL COMMENT '删除标记',
    `enabled_mark`      int          NOT NULL COMMENT '有效标志',
    `sort_code`         int          NULL DEFAULT NULL COMMENT '序号',

    `grade`             varchar(255) NOT NULL COMMENT '年级',
    `class_name`        varchar(255) NOT NULL COMMENT '班级',
    `major_name`        varchar(255) NOT NULL COMMENT '专业名称',
    `course_name`       varchar(255) NOT NULL COMMENT '课程名称',
    `course_code`       varchar(255) NOT NULL COMMENT '课程编号',
    `course_type`       varchar(255) NOT NULL COMMENT '课程分类',
    `teacher_id`        bigint       NOT NULL COMMENT '任课教师id',
    `teacher_name`      varchar(255) NOT NULL COMMENT '任课教师',
    `course_evaluation` varchar(255) NOT NULL COMMENT '课程质量评价',
    `student_number`    int          NOT NULL COMMENT '参评学生数',
    `evaluation_date`   date         NOT NULL COMMENT '评价完成日期',
    `academic_year` VARCHAR(9) NOT NULL COMMENT '学年（度），格式：YYYY-YYYY，如：2001-2002',
    `semester_code` VARCHAR(16) NOT NULL COMMENT '学期码，XQDM（学期代码）',

    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT = '教学质量与评价';

-- 教室视频流维护
DROP TABLE IF EXISTS `teacher_video_streaming_maintenance`;
CREATE TABLE `teacher_video_streaming_maintenance`
(
    id                     bigint       not null comment '主键编号'
        primary key,
    create_user_id         bigint       null comment '创建人',
    create_date            datetime     null comment '创建时间',
    modify_user_id         bigint       null comment '修改人',
    modify_date            datetime     null comment '修改时间',
    delete_mark            int          not null comment '删除标记',
    enabled_mark           int          not null comment '有效标志',
    sort_code              int          null comment '序号',

    `classroom_number`     varchar(255) NULL DEFAULT NULL COMMENT '教室号',
    `video_id`             varchar(255) NULL DEFAULT NULL COMMENT '视频设备id',
    `video_device_name`    varchar(255) NULL DEFAULT NULL COMMENT '视频设备点位名称',
    `video_stream_address` varchar(255) NULL DEFAULT NULL COMMENT '视频流地址'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='教室视频流维护';

-- 网络课程在线学习记录
DROP TABLE IF EXISTS `inter_class_studyrecode`;
CREATE TABLE `inter_class_studyrecode`
(
    id                     bigint       not null comment '主键编号'
        primary key,
    create_user_id         bigint       null comment '创建人',
    create_date            datetime     null comment '创建时间',
    modify_user_id         bigint       null comment '修改人',
    modify_date            datetime     null comment '修改时间',
    delete_mark            int          not null comment '删除标记',
    enabled_mark           int          not null comment '有效标志',
    sort_code              int          null comment '序号',

    inter_class_name       varchar(255) not null comment '网络课程名称',
    inter_class_source     varchar(255) not null comment '网络课程来源',
    inter_class_plaform    varchar(255) not null comment '网络课程所属平台名称',
    teacher_id             bigint       not null comment '主讲教师id',
    teacher_oa             varchar(255) not null comment '主讲教师工号',
    teacher_name           varchar(255) not null comment '主讲教师姓名',
    student_study_data     date         not null comment '学生学习日期',
    student_study_timesize double       not null comment '学生学习时长',
    student_study_number   int          not null comment '学生学习人次',
    student_submit_number  int          not null comment '学生提交作业数',
    teach_number           int          not null comment '教学资源数',
    answer_number          int          not null comment '答疑讨论数',
    input_people_id        bigint       null comment '录入人id',
    input_people           varchar(255) null comment '录入人',
    input_data             date         not null comment '录入日期'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 comment '网络课程在线学习记录';


-- 计划课程管理表
DROP TABLE IF EXISTS plan_course_manage;
CREATE TABLE plan_course_manage
(
    id                          bigint       not null comment '主键编号' primary key,
    create_user_id              bigint       null comment '创建人',
    create_date                 datetime     null comment '创建时间',
    modify_user_id              bigint       null comment '修改人',
    modify_date                 datetime     null comment '修改时间',
    delete_mark                 int          not null comment '删除标记',
    enabled_mark                int          not null comment '有效标志',
    sort_code                   int          null comment '序号',
    base_semester_id            bigint       not null comment '学期主键id(base_semester)',
    base_semester               varchar(256) not null comment '学期',
    course_plan_code            varchar(256) not null comment '计划课程编号',
    base_course_subject_id      bigint       not null comment '课程id(base_course_subject)',
    base_course_subject_name    varchar(256) not null comment '课程学科名称(base_course_subject)',
    base_course_subject_code    varchar(256) not null comment '课程学科代码(base_course_subject)',
    core_course                 varchar(32)  not null comment '是否专业核心课程(base_course_subject)',
    textbook_publication_number longtext     not null comment '教材出版号',
    teaching_method             varchar(256) not null comment '主要授课方式',
    exam_method                 varchar(32)  not null comment '考试方式',
    teacher_name                varchar(256) not null comment '主讲教师',
    teacher_id                  bigint       not null comment '主讲教师id',
    teacher_user_name           varchar(256) not null comment '主讲教师工号',
    course_certificate          varchar(32)  not null comment '是否课证融通课程',
    cert_id                     longtext     null comment '证书id',
    contest_name                longtext     null comment '相关竞赛名称',
    `semester_code` varchar(15) not null COMMENT '学期编号',
    `academic_year` VARCHAR(64) COMMENT '学年'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='计划课程管理表';

-- 学校成果管理
DROP TABLE IF EXISTS `base_school_outcome`;
CREATE TABLE base_school_outcome
(
    id              bigint       not null comment '主键编号' primary key,
    create_user_id  bigint       null comment '创建人',
    create_date     datetime     null comment '创建时间',
    modify_user_id  bigint       null comment '修改人',
    modify_date     datetime     null comment '修改时间',
    delete_mark     int          not null comment '删除标记',
    enabled_mark    int          not null comment '有效标志',
    sort_code       int          null comment '序号',
-- 共有属性
    outcome_name    VARCHAR(256) not null comment '成果名称',
    outcome_type    VARCHAR(16)  not null comment '成果类型',
    completion_unit VARCHAR(256) not null comment '完成单位',
    completion_date date         not null comment '完成时间',
    completer       VARCHAR(256) not null comment '完成人',
    overview        LONGTEXT     null comment '概述',
    file_id         LONGTEXT     not null comment '辅佐材料',
--     论文
    media_name      VARCHAR(256) null comment '媒体名称',
    media_type      VARCHAR(16)  null comment '媒体类型',
    issue_num_cn    VARCHAR(256) null comment '国内统一刊号CN',
    issue_num_intl  VARCHAR(256) null comment '国际标准刊号',
    journal_level   VARCHAR(32)  null comment '期刊级别',
    issued_date     date         null comment '发表时间',
--     专著
    monograph_type  VARCHAR(16)  null comment '专著类型',
    book_num_intl   VARCHAR(256) null comment '国际标准书号',
    publish_house   VARCHAR(256) null comment '出版社',
    publish_date    date         null comment '出版日期',
    edition         VARCHAR(128) null comment '版次',
--     专利
    patent_type     VARCHAR(64)  null comment '专利类型',
    patent_num      VARCHAR(256) null comment '专利号',
    patent_req      date         null comment '专利申请日期',
    patent_authz    date         null comment '专利授权日期',
    patentee        VARCHAR(256) null comment '专利权人',
--     软件著作权
    dev_finish_date date         null comment '开发完成日期',
    version_num     VARCHAR(64)  null comment '版本号',
    dev_status      VARCHAR(64)  null comment '开发情况',
--     教学成果奖
    teach_outcome   VARCHAR(64)  null comment '教学成果类型',
    innovation      LONGTEXT     null comment '创新点',
--     其它
    outcome_desc    LONGTEXT     null comment '成果说明'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='学校成果管理';


-- 实习计划管理表

DROP TABLE IF EXISTS internship_plan_manage;
CREATE TABLE internship_plan_manage
(
    id                                bigint       not null comment '主键编号' primary key,
    create_user_id                    bigint       null comment '创建人',
    create_date                       datetime     null comment '创建时间',
    modify_user_id                    bigint       null comment '修改人',
    modify_date                       datetime     null comment '修改时间',
    delete_mark                       int          not null comment '删除标记',
    enabled_mark                      int          not null comment '有效标志',
    sort_code                         int          null comment '序号',
    base_semester_id                  bigint       not null comment '学期主键id(base_semester)',
    base_semester                     varchar(256) not null comment '学期',
    internship_plan_name              varchar(256) not null comment '实习计划名称',
    internship_plan_type              varchar(256) not null comment '实习计划类型',
    training_base_id                  bigint       not null comment '实训基地主键id(school_training_base and outside_training_base)',
    training_base_name                varchar(256) not null comment '实训基地名称',
    professional_relevance_student    varchar(256) null comment '专业对口程度',
    internship_whereabouts            varchar(256) null comment '实习去向',
    insurance_purchaser               varchar(256) null comment '保险购买方',
    insurance_type                    varchar(256) null comment '购买保险种类',
    start_time                        date         not null comment '实习开始时间',
    end_time                          date         not null comment '实习结束时间',
    enterprise_industry               varchar(256) null comment '企业所属行业',
    internship_unit_name              varchar(256) not null comment '实习单位名称',
    internship_unit_address           varchar(256) null comment '实习单位地址',
    internship_unit_source            varchar(256) not null comment '实习单位来源',
    internship_job_name               varchar(256) not null comment '实习岗位名称',
    professional_relevance_enterprise varchar(256) null comment '专业对口程度',
    lodging_plan                      varchar(256) null comment '住宿安排',
    `internship_site_type` varchar(15) NOT NULL COMMENT '实习场所类型',
    `training_base_number` VARCHAR(255) COMMENT '实训基地编号',
    `academic_year`        VARCHAR(64) COMMENT '学年'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='实习计划管理表';

-- 实习计划参与人表
DROP TABLE IF EXISTS internship_plan_manage_participant;
CREATE TABLE internship_plan_manage_participant
(
    id                        bigint       not null comment '主键编号' primary key,
    create_user_id            bigint       null comment '创建人',
    create_date               datetime     null comment '创建时间',
    modify_user_id            bigint       null comment '修改人',
    modify_date               datetime     null comment '修改时间',
    delete_mark               int          not null comment '删除标记',
    enabled_mark              int          not null comment '有效标志',
    sort_code                 int          null comment '序号',
    internship_plan_manage_id bigint       not null comment '实习计划id',
    participant_id            bigint       null comment '实习参与人id',
    participant_user_id       varchar(256) null comment '实习参与人学号',
    participant_name          varchar(256) null comment '实习参与人姓名',
    base_major_name           varchar(255) not null comment '所属专业名称',
    base_major_id             bigint       not null comment '所属专业id(base_major)',
    class_name                varchar(256) not null comment '班级',
    class_id                  bigint       not null comment '班级id(base_class)'

) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='实习计划参与人表';

-- 党员管理数据表
DROP TABLE IF EXISTS `party_member_manage`;
CREATE TABLE party_member_manage
(
    id                bigint       not null comment '主键编号' primary key,
    create_user_id    bigint       null comment '创建人',
    create_date       datetime     null comment '创建时间',
    modify_user_id    bigint       null comment '修改人',
    modify_date       datetime     null comment '修改时间',
    delete_mark       int          not null comment '删除标记',
    enabled_mark      int          not null comment '有效标志',
    sort_code         int          null comment '序号',
    party_member_name VARCHAR(36)  not null comment '党员姓名',
    party_member_id   bigint       not null comment '党员编号',
    party_member_oa   VARCHAR(64)  not null comment '党员oa',
    party_tissue_name VARCHAR(256) not null comment '所属党组织名称',
    party_tissue_code bigint       not null comment '所属党组织编号',
    party_tissue_type varchar(256) not null comment '党组织类型',
    new_party_member  VARCHAR(25)  not null comment '是否新党员',
    member_type       VARCHAR(25)  null comment '新党员类型',
    growing_status    VARCHAR(25)  null comment '党员发展状态',
    activists_date    DATE         null comment '成为积极分子日期',
    prepare_member    DATE         null comment '成为预备党员日期',
    formal_member     DATE         null comment '成为正式党员日期',
    user_name  VARCHAR(25)  not null comment '党员学号(工号)'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='党员管理数据表';

-- 党组织管理
DROP TABLE IF EXISTS `party_tissue_manager`;
CREATE TABLE `party_tissue_manager`
(
    id                    bigint       not null comment '主键编号'
        primary key,
    create_user_id        bigint       null comment '创建人',
    create_date           datetime     null comment '创建时间',
    modify_user_id        bigint       null comment '修改人',
    modify_date           datetime     null comment '修改时间',
    delete_mark           int          not null comment '删除标记',
    enabled_mark          int          not null comment '有效标志',
    sort_code             int          null comment '序号',
    party_tissue_name     varchar(256) not null comment '党组织名称',
    party_tissue_code     varchar(256) not null comment '党组织编号',
    party_tissue_type     varchar(256) not null comment '党组织类型',
    superior_party_tissue varchar(256) null comment '隶属上级党组织',
    leadership_name       varchar(256) not null comment '党内领导姓名',
    leadership_number     varchar(256) null comment '党内领导工号',
    leadership_position   varchar(256) null comment '党内领导职务',
    party_counter         int          null comment '党组织党员数量'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    comment '党组织管理';

-- 证明申请
DROP TABLE IF EXISTS `prove_application`;
CREATE TABLE `prove_application`
(
    `id`                        bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`            bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`               datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`            bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`               datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`               int          NOT NULL COMMENT '删除标记',
    `enabled_mark`              int          NOT NULL COMMENT '有效标志',
    `sort_code`                 int          NULL DEFAULT NULL COMMENT '序号',
    `department_id`             bigint       NOT NULL comment '所在部门id',
    `department`                varchar(255) NOT NULL COMMENT '所在部门',
    department_head_id          bigint comment '部门主任id',
    department_head             varchar(255) comment '部门主任',
    executive_charge_id         bigint comment '分管领导id',
    executive_charge            varchar(255) comment '分管领导',
    `application_id`            bigint       NOT NULL COMMENT '申请人id',
    `application_name`          varchar(255) NULL DEFAULT NULL COMMENT '申请人',
    `application_time`          datetime     NULL DEFAULT NULL COMMENT '申请时间',
    `application_c_explanation` varchar(255) NULL DEFAULT NULL COMMENT '申请证明说明',
    `status`                    int          NOT NULL COMMENT '状态',
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='证明申请';


-- 报销记录表
DROP TABLE IF EXISTS `billing_reimbursement`;
create table billing_reimbursement
(
    id                   bigint         NOT NULL comment '主键编号'
        primary key,
    create_user_id       bigint         NULL comment '创建人',
    create_date          datetime       NULL comment '创建时间',
    modify_user_id       bigint         NULL comment '修改人',
    modify_date          datetime       NULL comment '修改时间',
    delete_mark          int            NOT NULL comment '删除标记',
    enabled_mark         int            NOT NULL comment '有效标志',
    sort_code            int            NULL comment '序号',
    department_id        bigint         NOT NULL comment '所在部门id',
    department_name      varchar(255)   NOT NULL comment '所在部门',
    applicant_id         bigint         NOT NULL comment '申请人id',
    applicant_name       varchar(255)   NOT NULL comment '申请人',
    department_manger_id bigint         NULL comment '所在部门主任id',
    department_head_id   bigint         NULL comment '所在分管领导id',
    aply_time            datetime       NOT NULL comment '申请时间',
    gotrip_time          date           NOT NULL comment '出差开始日期',
    end_time             date           NOT NULL comment '出差结束日期',
    start_end_o_postion  varchar(255)   NOT NULL comment '起讫地点',
    city_in              int            NOT NULL comment '市内市外',         -- 0为室内，1为市外
    business_trip_day    int            NOT NULL comment '天数',
    business_trip_reason varchar(512)   NOT NULL comment '出差事由',
    is_car               int            NOT NULL comment '是否派车',         -- 0为不派车，1为派车，
    city_traffic_fee     DECIMAL(10, 2) NULL comment '城市交通费',
    b_trip_traffic_fee   DECIMAL(10, 2) NULL comment '出差地交通费',
    is_lodging           int            NOT NULL comment '主办方是否包食宿', -- 0为不包，1为包
    food_fee             DECIMAL(10, 2) NULL comment '伙食补助费',
    lodging_fee          DECIMAL(10, 2) NULL comment '住宿费',
    total_amount         DECIMAL(10, 2) NOT NULL comment '报销费用合计（元)',
    file_id              longtext       NOT NULL COMMENT '附件',
    `status`             int            NOT NULL COMMENT '状态'
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    comment '报销记录表';


-- 社团活动表
DROP TABLE IF EXISTS `club_activities`;
CREATE TABLE `club_activities`
(
    `id`                      bigint       NOT NULL COMMENT '主键编号',
    `create_user_id`          bigint       NULL DEFAULT NULL COMMENT '创建人',
    `create_date`             datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `modify_user_id`          bigint       NULL DEFAULT NULL COMMENT '修改人',
    `modify_date`             datetime     NULL DEFAULT NULL COMMENT '修改时间',
    `delete_mark`             int          NOT NULL COMMENT '删除标记',
    `enabled_mark`            int          NOT NULL COMMENT '有效标志',
    `sort_code`               int          NULL DEFAULT NULL COMMENT '序号',
    `club_activities_name`    varchar(255) NOT NULL COMMENT '社团活动名称',
    `club_activities_content` text         NULL COMMENT '社团活动内容',
    `activity_type`           varchar(255) NOT NULL COMMENT '活动类型',
    `activity_location`       varchar(255) NULL DEFAULT NULL COMMENT '活动地点',
    `leader`                  varchar(255) NULL DEFAULT NULL COMMENT '负责人',
    `user_id`                 bigint       NOT NULL COMMENT '用户id',
    `student_name`            varchar(255) NOT NULL COMMENT '参与学生姓名',
    `student_id`              varchar(255) NOT NULL COMMENT '学号',
    `start_time`              datetime     NOT NULL COMMENT '活动开始时间',
    `end_time`                datetime     NULL DEFAULT NULL COMMENT '活动结束时间',
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4
    COMMENT = '社团活动表';

    -- 党员干部学习
DROP TABLE IF EXISTS `party_cadre_study`;
CREATE TABLE `party_cadre_study`
(
    id                        BIGINT       NOT NULL COMMENT '主键编号'
        PRIMARY KEY,
    create_user_id            BIGINT       NULL COMMENT '创建人',
    create_date               DATETIME     NULL COMMENT '创建时间',
    modify_user_id            BIGINT       NULL COMMENT '修改人',
    modify_date               DATETIME     NULL COMMENT '修改时间',
    delete_mark               INT          NOT NULL COMMENT '删除标记',
    enabled_mark              INT          NOT NULL COMMENT '有效标志',
    sort_code                 INT          NULL COMMENT '序号',
    party_tissue_id           BIGINT       NOT NULL COMMENT '党组织主键id(party_tissue_manager)',
    party_tissue_name         VARCHAR(256) NOT NULL COMMENT '党组织名称',
    party_tissue_code         VARCHAR(256) NOT NULL COMMENT '党组织编号',
    party_train_way_carrier   VARCHAR(500) NOT NULL COMMENT '党员干部学习培训主要途径和载体',
    party_train_way_content   VARCHAR(500) NOT NULL COMMENT '党员干部学习培训内容',
    participant_teacher_name  Longtext     null comment '活动参与人教师姓名',
    participant_student_name  Longtext     null comment '活动参与人学生姓名',
    start_time                DATE         NOT NULL COMMENT '活动开始日期',
    end_time                  DATE         NOT NULL COMMENT '活动结束日期',
    participate_teacher_count INT          NULL COMMENT '参与教师数',
    participate_student_count INT          NULL COMMENT '参与学生数'
) ENGINE = INNODB
  DEFAULT CHARSET = utf8mb4
    COMMENT '党员干部学习';

-- 党课管理数据表
DROP TABLE IF EXISTS party_class_manage;
CREATE TABLE party_class_manage
(
    id                        BIGINT       NOT NULL COMMENT '主键编号' PRIMARY KEY,
    create_user_id            BIGINT       NULL COMMENT '创建人',
    create_date               DATETIME     NULL COMMENT '创建时间',
    modify_user_id            BIGINT       NULL COMMENT '修改人',
    modify_date               DATETIME     NULL COMMENT '修改时间',
    delete_mark               INT          NOT NULL COMMENT '删除标记',
    enabled_mark              INT          NOT NULL COMMENT '有效标志',
    sort_code                 INT          NULL COMMENT '序号',
    party_tissue_name         VARCHAR(256) NOT NULL COMMENT '党组织名称',
    party_tissue_code         VARCHAR(256) NOT NULL COMMENT '党组织编号',
    party_tissue_id           BIGINT       NOT NULL COMMENT '党组织主键id(party_tissue_manager)',
    participant_teacher_name  Longtext     null comment '活动参与人教师姓名',
    participant_student_name  Longtext     null comment '活动参与人学生姓名',
    activity_type             varchar(256) null comment '活动形式',
    speaker_name              varchar(256) null comment '党课主讲人',
    start_time                DATE         NOT NULL COMMENT '活动开始时间',
    end_time                  DATE         NOT NULL COMMENT '活动结束时间',
    activities_content        TEXT         NULL COMMENT '活动内容',
    participate_teacher_count INT          NULL COMMENT '参与教师数',
    participate_student_count INT          NULL COMMENT '参与学生数'

) ENGINE = INNODB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='党课管理数据表'; 

-- 党建活动
DROP TABLE IF EXISTS `party_build_event`;
CREATE TABLE party_build_event
(
    id                       BIGINT       NOT NULL COMMENT '主键编号'
        PRIMARY KEY,
    create_user_id           BIGINT       NULL COMMENT '创建人',
    create_date              DATETIME     NULL COMMENT '创建时间',
    modify_user_id           BIGINT       NULL COMMENT '修改人',
    modify_date              DATETIME     NULL COMMENT '修改时间',
    delete_mark              INT          NOT NULL COMMENT '删除标记',
    enabled_mark             INT          NOT NULL COMMENT '有效标志',
    sort_code                INT          NULL COMMENT '序号',
    event_type               VARCHAR(255) NOT NULL COMMENT '活动类型',
    party_tissue_id          BIGINT       NOT NULL COMMENT '党组织主键id(party_tissue_manager)',
    party_tissue_name        VARCHAR(256) NOT NULL COMMENT '党组织名称',
    party_tissue_code        VARCHAR(256) NOT NULL COMMENT '党组织编号',
    participant_teacher_name Longtext     null comment '活动参与人教师姓名',
    participant_student_name Longtext     null comment '活动参与人学生姓名',
    event_address            VARCHAR(255) NULL COMMENT '活动地点',
    event_content            TEXT         NULL COMMENT '活动内容',
    event_start_time         DATE         NOT NULL COMMENT '活动开始时间',
    event_end_time           DATE         NOT NULL COMMENT '活动结束时间',
    attend_teacher_num       INT          NULL COMMENT '参于教师数',
    attend_student_num       INT          NULL COMMENT '参于学生数'
) ENGINE = INNODB
  DEFAULT CHARSET = utf8mb4
    COMMENT ='党建活动';










